/*****************************
* Source : structure.prg           
* System : <unkown>           
* Author : Phil Ide           
* Created: 15/05/2003             
*                             
* Purpose:                    
* ----------------------------
* History:                    
* ----------------------------
*    15/05/2003 14:11 PPI - Created                
*****************************/

#include "common.ch"
#include "class.ch"
#include "error.ch"

CLASS Structure
   EXPORTED:
      VAR ___name
      VAR ___parent
      VAR message

      METHOD init
      #if XPPVER < 1900000
         ACCESS ASSIGN METHOD NoIvarCallBack
      #else
         METHOD getNoIvar
         METHOD setNoIvar
      #endif

      METHOD get
      METHOD put

      METHOD raw IS get
      METHOD sizeOf

      METHOD getname
      METHOD setParent

   PROTECTED:
      VAR aItem

      METHOD NoIvar
      METHOD addItem

ENDCLASS

METHOD Structure:init()
   ::aItem := {}
   return self

METHOD Structure:getName()
   local cRet := ::classname()
   if !(::___name == NIL)
      cRet := ::___name
   endif
   return cRet

METHOD Structure:addItem(o, name)
   aadd(::aItem, o)
   o:setParent(self)
   if !(name == NIL) .and. o:IsDerivedFrom(Structure())
      o:___name := name
   endif
   return len(::aItem)

METHOD Structure:setParent(o)
   ::___parent := o
   return self

#if XPPVER < 1900000
   METHOD Structure:NoIvarCallBack(x)
      local i
      local xRet

      i := AScan(::aItem, {|e| lower(e:getName()) == lower(::message) } )
      if i > 0
         if !(x == NIL)
            if ::aItem[i]:isDerivedFrom('Structure') .or. ::aItem[i]:isDerivedFrom('Union')
               ::aItem[i]:&(::message) := x // we should never get here, but this code will
                                            // cause an error later, so I've left it.
            else
               xRet := ::aItem[i]:put(x, ::message)
            endif
         else
            if ::aItem[i]:isDerivedFrom('Structure') .or. ::aItem[i]:isDerivedFrom('Union')
               xRet := ::aItem[i]//:&(::message)
            else
               xRet := ::aItem[i]:fetch(::message)
            endif
         endif
      else
         ::noIvar(x)
      endif
      return xRet
#else
   METHOD Structure:getNoIVar(x)
   local i
   local xRet

   i := AScan(::aItem, {|e| lower(e:getName()) == lower(x) } )
   if i > 0
      if ::aItem[i]:isDerivedFrom('Structure') .or. ::aItem[i]:isDerivedFrom('Union')
         xRet := ::aItem[i]
      else
         xRet := ::aItem[i]:fetch(::message)
      endif
   else
      ::message := x
      ::noIvar()
   endif
   return xRet

   METHOD Structure:setNoIVar(cVar, xValue)
   local i
   local xRet

   i := AScan(::aItem, {|e| lower(e:getName()) == lower(cVar) } )
   if i > 0
      if ::aItem[i]:isDerivedFrom('Structure') .or. ::aItem[i]:isDerivedFrom('Union')
         ::aItem[i]:&(cVar) := xValue // we should never get here, but this code will
                                      // cause an error later, so I've left it.
      else
         xRet := ::aItem[i]:put(cVar, xValue)
      endif
   else
      ::message := cVar
      ::noIvar(xValue)
   endif
   return xRet
#endif

METHOD Structure:NoIvar( x )
   local o := Error():new()

   o:args            := x
   o:canDefault      := FALSE
   o:canRetry        := FALSE
   o:canSubstitute   := FALSE
   o:description     := "invalid structure element name"
   o:operation       := ::className()+":"+::message
   o:severity        := XPP_ES_ERROR
   o:subSystem       := ::className()
   o:thread          := ThreadID()
   Eval( ErrorBlock(), o )
   return self

METHOD Structure:get()
   local i
   local cBuff := ''

   for i := 1 to len(::aItem)
      cBuff += ::aItem[i]:raw()
   next
   return cBuff

METHOD Structure:put(cBuff)
   local i
   local nStart := 1
   local nEnd

   for i := 1 to len(::aItem)
      nEnd := len(::aItem[i]:raw())
      if nStart+nEnd <= Len(cBuff)+1
         ::aItem[i]:rawPut( SubStr(cBuff, nStart, nEnd) )
         nStart += nEnd
      else
         exit
      endif
   next
   return self

METHOD Structure:sizeOf()
   return Len(::get())

