/* syntax:

   STRUCTURE <structName>
       type <memberName> [SIZE <nSize>]
       [type <memberName> [SIZE <nSize>]]
   ENDSTRUCT

   "type" can be:
      WORD
      DWORD
      HANDLE
      INT
      UINT
      SHORT
      USHORT
      LONG
      ULONG
      CHAR

      If type=CHAR, then you can specify SIZE to indocate the length of
      the member.  If specified, the length will *always* be the same,
      regardless of the content (it will be padded to the right if too short,
      or if too long an error will occur).

      ENDSTRUCT is a pseudonym for ENDSTRUCTURE

      To instantiate a class:

      STRUCTURE myStruct
         LONG id
         CHAR foreName
         CHAR lastName
         SHORT age
      ENDSTRUCT

      Function Test()
         local oMy := myStruct():new()

         oMy:id       := 1
         oMy:foreName := "Fred"
         oMy:lastname := "Bloggs"
         oMy:age      := 21


   To assist identifying this object as a structure in your code, or to aid in
   converting C/C++/VB/Delphi code, you can create translation directives:

   #xtranslate .id => :id
   #xtranslate .forename => :foreName
   #xtranslate .lastName => :lastName
   #xtranslate .age      => :age


         oMy.id       := 1
         oMy.foreName := "Fred"
         oMy.lastname := "Bloggs"
         oMy.age      := 21


   Note that you can only create the translation once, so if you have several
   structures with members with the same names (e.g. .age ocurs in two different
   structures), you must ensure that you only create one translation directive.
*/

// data types by group
#define DWORD  LONG
#define HANDLE LONG
#define SHORT  INT
#define USHORT  UINT
#define WORD   UINT

#xtranslate STRUCTURE <struct> => ;
   CLASS <struct> FROM Structure;
;      EXPORTED:;
;         VAR cargo;
;         METHOD init;
;   ENDCLASS;
;   METHOD <struct>:init();
;      ::Structure:init()



#xtranslate LONG <member> [<ref:BYREF>] => ;
      ::addItem( StructItemLong():new(<(member)>,,<.ref.>))

#xtranslate ULONG <member> [<ref:BYREF>] => ;
      ::addItem( StructItemULong():new(<(member)>,,<.ref.>))

#xtranslate INT <member> [<ref:BYREF>] => ;
      ::addItem( StructItemInt():new(<(member)>,,<.ref.>))

#xtranslate UINT <member> [<ref:BYREF>] => ;
      ::addItem( StructItemUInt():new(<(member)>,,<.ref.>))

#xtranslate CHAR <member> [SIZE <nSize>] [<ref:BYREF>] => ;
      ::addItem( StructItemBChar():new(<(member)>,<nSize>,<.ref.>))

#xtranslate STRUCT <member> AS <structure> => ;
      ::addItem( <structure>():new(), #<member>)


#xtranslate ENDSTRUCT => return self
#xtranslate ENDSTRUCTURE => return self

#xtranslate .cargo => :cargo

